import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

  if (world.scoreboard.getObjective("settings") == undefined) {
    world.scoreboard.addObjective("settings", "settings");
  }

const settingsOrder = [
  "leafFallRate",
  "blowBubbles",
];

const defaultValues = {
  leafFallRate: 5,
  blowBubbles: 1,
};

if (world.scoreboard.getObjective("settings") == undefined) {
  world.scoreboard.addObjective("settings", "settings");
}

settingsOrder.forEach((setting) => {
  const objective = world.scoreboard.getObjective("settings");
  try {
    objective.getScore(setting);
  } catch (error) {
    // Set the default value if an error occurs
    const defaultValue = defaultValues[setting] || 0; // Use 0 if not specified
    objective.setScore(setting, defaultValue);
  }
});

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_tpa:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_pretty_particles:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});

system.runInterval(() =>{
  for (let i = 0; i < world.scoreboard.getObjective("settings").getScore("leafFallRate"); i++) {
    for (const player of world.getAllPlayers()) {
      const checkBlock = player.dimension.getBlock(player.location).offset({x: ((Math.random() * 20) - 10), y: ((Math.random() * 12) + 1), z: ((Math.random() * 20) - 10)});
      const blockType = checkBlock.typeId;
      switch (blockType) {
        case "minecraft:oak_leaves":
            player.dimension.spawnParticle("sg_pretty_particles:oak_leaf", checkBlock.location);
            break;
        case "minecraft:spruce_leaves":
            player.dimension.spawnParticle("sg_pretty_particles:spruce_leaf", checkBlock.location);
            break;
        case "minecraft:acacia_leaves":
            player.dimension.spawnParticle("sg_pretty_particles:acacia_leaf", checkBlock.location);
            break;
        case "minecraft:mangrove_leaves":
            player.dimension.spawnParticle("sg_pretty_particles:mangrove_leaf", checkBlock.location);
            break;
        case "minecraft:jungle_leaves":
            player.dimension.spawnParticle("sg_pretty_particles:jungle_leaf", checkBlock.location);
            break;
        case "minecraft:birch_leaves":
            player.dimension.spawnParticle("sg_pretty_particles:birch_leaf", checkBlock.location);
            break;
        case "minecraft:cherry_leaves":
            player.dimension.spawnParticle("sg_pretty_particles:cherry_leaf", checkBlock.location);
            break;
        case "minecraft:dark_oak_leaves":
            // Exception for dark oak leaves, spawns oak leaf particle instead
            player.dimension.spawnParticle("sg_pretty_particles:oak_leaf", checkBlock.location);
            break;
        default:
            // Optional: handle other cases if needed
            break;
      }
    }
  }
},1)

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    if (world.scoreboard.getObjective("settings").getScore("blowBubbles") && player.dimension.getBlock(player.getHeadLocation()).isLiquid) player.dimension.spawnParticle("minecraft:dolphin_move_particle", player.getHeadLocation())
  }
}, 20)










world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_pretty_particles_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_pretty_particles_version", VERSION);
    if (player.hasTag("sg_pretty_particles_received_guide")) {
      player.removeTag("sg_pretty_particles_received_guide");
      player.sendMessage({
        translate: "sg_pretty_particles.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_pretty_particles:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_pretty_particles_version", VERSION);
    player.sendMessage({
      translate: "sg_pretty_particles.announce_update",
      with: [VERSION],
    });
  }
});



const BUTTONS = [
  {
    // OPTIONS
    translation_key: "sg_pretty_particles.guide.options.title",
    icon_path: "textures/ui/book_edit_default.png",
    action: (player) => {

      const objective = world.scoreboard.getObjective("settings");

      // Create and show the modal form
      const form = new ModalFormData()
        .title("Options")
        .slider(
          "Falling Leaves Rate",
          0,
          20,
          1,
          world.scoreboard.getObjective("settings").getScore("leafFallRate")
        )
        .toggle(
          "Blow Bubbles",
          Boolean(world.scoreboard.getObjective("settings").getScore("blowBubbles"))
        )
        .submitButton("Save Options");

      form.show(player).then((res) => {
        if (res.canceled) return;

        // Iterate over form values and map them to the settings labels
        const settings = res.formValues;
        settings.forEach((value, index) => {
          const label = settingsOrder[index];
          if (label) {
            objective.setScore(label, value);
          }
        });
        world.sendMessage("§dPretty Particles options have been updated.");
        if (
          world.scoreboard
            .getObjective("settings")
            .getScore("leafFallRate") > 13
        ) {
          world.sendMessage(
            "§cSetting the Falling Leaves Rate to a high value can cause lag on lower-end devices."
          );
        }
      });
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_pretty_particles.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_pretty_particles.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_pretty_particles.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_pretty_particles.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_pretty_particles.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_pretty_particles.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_pretty_particles.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_pretty_particles:guide") {
      showMainPage(event.source);
    }
  });
});